@@ -1,3 +1,9 @@
+# Copyright (c) 2018 The Pooch Developers.
+# Distributed under the terms of the BSD 3-Clause License.
+# SPDX-License-Identifier: BSD-3-Clause
+#
+# This code is part of the Fatiando a Terra project (https://www.fatiando.org)
+#
 """
 Test the core class and factory function.
 """
@@ -9,11 +15,17 @@ from tempfile import TemporaryDirectory
 import pytest
 
 from ..core import Pooch, retrieve, download_action, stream_download
-from ..utils import file_hash, get_logger, temporary_file, os_cache
+from ..utils import get_logger, temporary_file, os_cache
+from ..hashes import file_hash, hash_matches
+
+# Import the core module so that we can monkeypatch some functions
+from .. import core
 from ..downloaders import HTTPDownloader
 
 from .utils import (
     pooch_test_url,
+    pooch_test_figshare_url,
+    pooch_test_zenodo_url,
     pooch_test_registry,
     check_tiny_data,
     check_large_data,
@@ -25,6 +37,8 @@ ON_TRAVIS = bool(os.environ.get("TRAVIS", None))
 DATA_DIR = str(Path(__file__).parent / "data")
 REGISTRY = pooch_test_registry()
 BASEURL = pooch_test_url()
+FIGSHAREURL = pooch_test_figshare_url()
+ZENODOURL = pooch_test_zenodo_url()
 REGISTRY_CORRUPTED = {
     # The same data file but I changed the hash manually to a wrong one
     "tiny-data.txt": "098h0894dba14b12085eacb204284b97e362f4f3e5a5807693cc90ef415c1b2d"
@@ -106,11 +120,14 @@ def test_pooch_local():
     check_tiny_data(fname)
 
 
-def test_pooch_custom_url():
+@pytest.mark.parametrize(
+    "url", [BASEURL, FIGSHAREURL, ZENODOURL], ids=["https", "figshare", "zenodo"]
+)
+def test_pooch_custom_url(url):
     "Have pooch download the file from URL that is not base_url"
     with TemporaryDirectory() as local_store:
         path = Path(local_store)
-        urls = {"tiny-data.txt": BASEURL + "tiny-data.txt"}
+        urls = {"tiny-data.txt": url + "tiny-data.txt"}
         # Setup a pooch in a temp dir
         pup = Pooch(path=path, base_url="", registry=REGISTRY, urls=urls)
         # Check that the logs say that the file is being downloaded
@@ -118,7 +135,7 @@ def test_pooch_custom_url():
             fname = pup.fetch("tiny-data.txt")
             logs = log_file.getvalue()
             assert logs.split()[0] == "Downloading"
-            assert logs.split()[-1] == "'{}'.".format(path)
+            assert logs.split()[-1] == f"'{path}'."
         check_tiny_data(fname)
         # Check that no logging happens when there are no events
         with capture_log() as log_file:
@@ -126,19 +143,22 @@ def test_pooch_custom_url():
             assert log_file.getvalue() == ""
 
 
-def test_pooch_download():
+@pytest.mark.parametrize(
+    "url", [BASEURL, FIGSHAREURL, ZENODOURL], ids=["https", "figshare", "zenodo"]
+)
+def test_pooch_download(url):
     "Setup a pooch that has no local data and needs to download"
     with TemporaryDirectory() as local_store:
         path = Path(local_store)
         true_path = str(path / "tiny-data.txt")
         # Setup a pooch in a temp dir
-        pup = Pooch(path=path, base_url=BASEURL, registry=REGISTRY)
+        pup = Pooch(path=path, base_url=url, registry=REGISTRY)
         # Check that the logs say that the file is being downloaded
         with capture_log() as log_file:
             fname = pup.fetch("tiny-data.txt")
             logs = log_file.getvalue()
             assert logs.split()[0] == "Downloading"
-            assert logs.split()[-1] == "'{}'.".format(path)
+            assert logs.split()[-1] == f"'{path}'."
         # Check that the downloaded file has the right content
         assert true_path == fname
         check_tiny_data(fname)
@@ -149,6 +169,105 @@ def test_pooch_download():
             assert log_file.getvalue() == ""
 
 
+class FakeHashMatches:  # pylint: disable=too-few-public-methods
+    "Create a fake version of hash_matches that fails n times"
+
+    def __init__(self, nfailures):
+        self.nfailures = nfailures
+        self.failed = 0
+
+    def hash_matches(self, *args, **kwargs):
+        "Fail n times before finally passing"
+        if self.failed < self.nfailures:
+            self.failed += 1
+            # Give it an invalid hash to force a failure
+            return hash_matches(args[0], "bla", **kwargs)
+        return hash_matches(*args, **kwargs)
+
+
+def test_pooch_download_retry_off_by_default(monkeypatch):
+    "Check that retrying the download is off by default"
+    with TemporaryDirectory() as local_store:
+        monkeypatch.setattr(core, "hash_matches", FakeHashMatches(3).hash_matches)
+        # Setup a pooch without download retrying
+        path = Path(local_store)
+        pup = Pooch(path=path, base_url=BASEURL, registry=REGISTRY)
+        # Make sure it fails with no retries
+        with pytest.raises(ValueError) as error:
+            with capture_log() as log_file:
+                pup.fetch("tiny-data.txt")
+        assert "does not match the known hash" in str(error)
+        # Check that the log doesn't have the download retry message
+        logs = log_file.getvalue().strip().split("\n")
+        assert len(logs) == 1
+        assert logs[0].startswith("Downloading")
+        assert logs[0].endswith(f"'{path}'.")
+
+
+class FakeSleep:  # pylint: disable=too-few-public-methods
+    "Create a fake version of sleep that logs the specified times"
+
+    def __init__(self):
+        self.times = []
+
+    def sleep(self, secs):
+        "Store the time and doesn't sleep"
+        self.times.append(secs)
+
+
+def test_pooch_download_retry(monkeypatch):
+    "Check that retrying the download works if the hash is different"
+    with TemporaryDirectory() as local_store:
+        monkeypatch.setattr(core, "hash_matches", FakeHashMatches(11).hash_matches)
+        fakesleep = FakeSleep()
+        monkeypatch.setattr(core.time, "sleep", fakesleep.sleep)
+        # Setup a pooch with download retrying
+        path = Path(local_store)
+        true_path = str(path / "tiny-data.txt")
+        retries = 11
+        pup = Pooch(
+            path=path, base_url=BASEURL, registry=REGISTRY, retry_if_failed=retries
+        )
+        # Check that the logs say that the download failed n times
+        with capture_log() as log_file:
+            fname = pup.fetch("tiny-data.txt")
+            logs = log_file.getvalue().strip().split("\n")
+            assert len(logs) == 1 + retries
+            assert logs[0].startswith("Downloading")
+            assert logs[0].endswith(f"'{path}'.")
+            for i, line in zip(range(retries, 0, -1), logs[1:]):
+                assert "Failed to download" in line
+                plural = "s" if i > 1 else ""
+                assert f"download again {i} more time{plural}." in line
+            # Check that the sleep time increases but stops at 10s
+            assert fakesleep.times == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 10]
+        # Check that the downloaded file has the right content
+        assert true_path == fname
+        check_tiny_data(fname)
+        assert file_hash(fname) == REGISTRY["tiny-data.txt"]
+
+
+def test_pooch_download_retry_fails_eventually(monkeypatch):
+    "Check that retrying the download fails after the set amount of retries"
+    with TemporaryDirectory() as local_store:
+        monkeypatch.setattr(core, "hash_matches", FakeHashMatches(3).hash_matches)
+        # Setup a pooch with insufficient retry attempts
+        path = Path(local_store)
+        pup = Pooch(path=path, base_url=BASEURL, registry=REGISTRY, retry_if_failed=1)
+        # Make sure it fails with no retries
+        with pytest.raises(ValueError) as error:
+            # Check that the logs say that the download failed n times
+            with capture_log() as log_file:
+                pup.fetch("tiny-data.txt")
+        logs = log_file.getvalue().strip().split("\n")
+        assert len(logs) == 2
+        assert logs[0].startswith("Downloading")
+        assert logs[0].endswith(f"'{path}'.")
+        assert "Failed to download" in logs[1]
+        assert "download again 1 more time." in logs[1]
+        assert "does not match the known hash" in str(error)
+
+
 def test_pooch_logging_level():
     "Setup a pooch and check that no logging happens when the level is raised"
     with TemporaryDirectory() as local_store:
@@ -179,7 +298,7 @@ def test_pooch_update():
             fname = pup.fetch("tiny-data.txt")
             logs = log_file.getvalue()
             assert logs.split()[0] == "Updating"
-            assert logs.split()[-1] == "'{}'.".format(path)
+            assert logs.split()[-1] == f"'{path}'."
         # Check that the updated file has the right content
         assert true_path == fname
         check_tiny_data(fname)
@@ -202,7 +321,7 @@ def test_pooch_corrupted():
             assert "(tiny-data.txt)" in str(error.value)
             logs = log_file.getvalue()
             assert logs.split()[0] == "Downloading"
-            assert logs.split()[-1] == "'{}'.".format(path)
+            assert logs.split()[-1] == f"'{path}'."
     # and the case where the file exists but hash doesn't match
     pup = Pooch(path=DATA_DIR, base_url=BASEURL, registry=REGISTRY_CORRUPTED)
     with capture_log() as log_file:
@@ -211,7 +330,7 @@ def test_pooch_corrupted():
         assert "(tiny-data.txt)" in str(error.value)
         logs = log_file.getvalue()
         assert logs.split()[0] == "Updating"
-        assert logs.split()[-1] == "'{}'.".format(DATA_DIR)
+        assert logs.split()[-1] == f"'{DATA_DIR}'."
 
 
 def test_pooch_file_not_in_registry():
@@ -231,6 +350,14 @@ def test_pooch_load_registry():
     assert pup.registry_files.sort() == list(REGISTRY).sort()
 
 
+def test_pooch_load_registry_comments():
+    "Loading the registry from a file and strip line comments"
+    pup = Pooch(path="", base_url="")
+    pup.load_registry(os.path.join(DATA_DIR, "registry_comments.txt"))
+    assert pup.registry == REGISTRY
+    assert pup.registry_files.sort() == list(REGISTRY).sort()
+
+
 def test_pooch_load_registry_fileobj():
     "Loading the registry from a file object"
     path = os.path.join(DATA_DIR, "registry.txt")
@@ -244,7 +371,7 @@ def test_pooch_load_registry_fileobj():
 
     # Text mode
     pup = Pooch(path="", base_url="")
-    with open(path, "r") as fin:
+    with open(path) as fin:
         pup.load_registry(fin)
     assert pup.registry == REGISTRY
     assert pup.registry_files.sort() == list(REGISTRY).sort()
@@ -287,13 +414,13 @@ def test_check_availability_on_ftp():
     # Check available remote file on FTP server
     pup = Pooch(
         path=DATA_DIR,
-        base_url="ftp://speedtest.tele2.net/",
+        base_url="ftp://data-out.unavco.org/pub/products/velocity/rel_201712/",
         registry={
-            "100KB.zip": "f627ca4c2c322f15db26152df306bd4f983f0146409b81a4341b9b340c365a16",
+            "pbo.final_igs08.20171202.vel": "md5:0b75d4049dedd0e179615f4b5e956156",
             "doesnot_exist.zip": "jdjdjdjdflld",
         },
     )
-    assert pup.is_available("100KB.zip")
+    assert pup.is_available("pbo.final_igs08.20171202.vel")
     # Check non available remote file
     assert not pup.is_available("doesnot_exist.zip")
 
@@ -348,7 +475,7 @@ def test_alternative_hashing_algorithms():
     for alg in ("sha512", "md5"):
         hasher = hashlib.new(alg)
         hasher.update(data)
-        registry = {"tiny-data.txt": "{}:{}".format(alg, hasher.hexdigest())}
+        registry = {"tiny-data.txt": f"{alg}:{hasher.hexdigest()}"}
         pup = Pooch(path=DATA_DIR, base_url="some bogus URL", registry=registry)
         assert fname == pup.fetch("tiny-data.txt")
         check_tiny_data(fname)
